"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var merge_1 = require("lodash-es/merge");
var Spinner = require("spin");
var ngx_angular_loading_service_1 = require("./ngx-angular-loading.service");
var NGXLoadingDirective = (function () {
    function NGXLoadingDirective(renderer, ngxLoadingService, myElement) {
        this.renderer = renderer;
        this.ngxLoadingService = ngxLoadingService;
        this.myElement = myElement;
        this.key = false;
        this.spinner = null;
    }
    NGXLoadingDirective.prototype.ngOnDestroy = function () {
        this.startSubscription.unsubscribe();
        this.finishSubscription.unsubscribe();
        this.finish();
        this.spinner = null;
    };
    NGXLoadingDirective.prototype.ngOnChanges = function (inputs) {
        var _this = this;
        if (inputs.loadingOptions) {
            if (!inputs.loadingOptions.isFirstChange()) {
                this.finish();
            }
            this.options = merge_1.default({}, this.ngxLoadingService.ngxLoadingOptions, inputs.loadingOptions.currentValue);
            this.body = this.newDiv('ngx-loading-body');
            this.container = this.newDiv('ngx-loading');
            this.renderer.appendChild(this.container, this.body);
            if (this.options.overlay) {
                this.renderer.addClass(this.container, 'ngx-loading-overlay');
            }
            if (this.options.className) {
                this.renderer.addClass(this.container, this.options.className);
            }
            if (this.options.spinner) {
                this.spinnerContainer = this.newDiv('ngx-loading-spinner');
                this.renderer.appendChild(this.body, this.spinnerContainer);
                this.spinner = new Spinner(this.options.spinnerOptions);
            }
            if (this.options.text) {
                this.text = this.newDiv('ngx-loading-text');
                var mytext = this.renderer.createText(this.options.text);
                this.renderer.appendChild(this.text, mytext);
                this.renderer.appendChild(this.body, this.text);
            }
            this.renderer.appendChild(this.myElement.nativeElement, this.container);
            if (this.options.active || !this.key) {
                this.start();
            }
            if (inputs.loadingOptions.isFirstChange()) {
                this.startSubscription = this.ngxLoadingService.ngxLoadingStart$
                    .subscribe(function (item) {
                    if (item === _this.key) {
                        _this.start();
                    }
                });
                this.finishSubscription = this.ngxLoadingService.ngxLoadingFinish$
                    .subscribe(function (item) {
                    if (item === _this.key) {
                        _this.finish();
                    }
                });
            }
        }
    };
    NGXLoadingDirective.prototype.newDiv = function (className) {
        var change_this;
        change_this = this.renderer.createElement('div');
        this.renderer.addClass(change_this, className);
        return change_this;
    };
    NGXLoadingDirective.prototype.start = function () {
        if (this.container) {
            this.renderer.addClass(this.container, 'ngx-loading-active');
        }
        if (this.spinner) {
            this.spinner.spin(this.spinnerContainer);
        }
    };
    NGXLoadingDirective.prototype.finish = function () {
        if (this.container) {
            this.renderer.removeClass(this.container, 'ngx-loading-active');
        }
        if (this.spinner) {
            this.spinner.stop();
        }
    };
    NGXLoadingDirective.decorators = [
        { type: core_1.Directive, args: [{
                    selector: '[ngxLoading]'
                },] },
    ];
    NGXLoadingDirective.ctorParameters = function () { return [
        { type: core_1.Renderer2, },
        { type: ngx_angular_loading_service_1.NGXLoadingService, },
        { type: core_1.ElementRef, },
    ]; };
    NGXLoadingDirective.propDecorators = {
        "key": [{ type: core_1.Input, args: ['ngxLoading',] },],
        "loadingOptions": [{ type: core_1.Input },],
    };
    return NGXLoadingDirective;
}());
exports.NGXLoadingDirective = NGXLoadingDirective;
//# sourceMappingURL=ngx-angular-loading.component.js.map